@@ -5,6 +5,20 @@ import textwrap
 from decimal import Decimal
 
 
+def normalize_layout_string(layout_string):
+    """normalize a layout string such that it is characterwise identical
+    to the output of the 'dump' command
+    """
+    layout_string = layout_string.replace('\n', ' ')
+    # drop multiple whitespace
+    layout_string = re.sub(r'[ ]+', ' ', layout_string)
+    # drop whitespace after opening parenthesis
+    layout_string = re.sub(r'[(] ', r'(', layout_string)
+    # drop whitespace and before closing parenthesis
+    layout_string = re.sub(r' [)]', r')', layout_string)
+    return layout_string.strip()
+
+
 def verify_frame_objects_via_dump(hlwm, tag_object="tags.focus"):
     """verify, that the frame objects for the given tag_object match
     with the output of the 'dump' command this tag"""
@@ -82,6 +96,33 @@ def test_explode(hlwm, running_clients, running_clients_num):
     verify_frame_objects_via_dump(hlwm)
 
 
+def test_explode_second_client(hlwm):
+    winids = hlwm.create_clients(2)
+    hlwm.call(['load', f'(clients vertical:1 {winids[0]} {winids[1]})'])
+    hlwm.call('split explode')
+
+    assert hlwm.get_attr('tags.0.curframe_wcount') == '1'
+    assert hlwm.get_attr('tags.0.curframe_windex') == '0'
+    expected_layout = normalize_layout_string(f"""\
+    (split vertical:0.5:1
+    (clients vertical:0 {winids[0]})
+    (clients vertical:0 {winids[1]}))
+    """)
+    assert hlwm.call('dump').stdout == expected_layout
+    verify_frame_objects_via_dump(hlwm)
+
+
+@pytest.mark.parametrize("running_clients_num", [4])
+@pytest.mark.parametrize("focus_idx", range(0, 4))
+def test_explode_preserves_focus(hlwm, running_clients, running_clients_num, focus_idx):
+    hlwm.call(['focus_nth', focus_idx])
+    winid = hlwm.attr.clients.focus.winid()
+
+    for i in range(0, running_clients_num):
+        hlwm.call('split explode')
+        assert winid == hlwm.attr.clients.focus.winid()
+
+
 @pytest.mark.parametrize("running_clients_num", [0, 1, 4])
 def test_remove(hlwm, running_clients, running_clients_num):
     hlwm.call('split explode')
@@ -155,6 +196,63 @@ def test_focus_wrap(hlwm, running_clients, running_clients_num):
             hlwm.call('focus down')
 
 
+@pytest.mark.parametrize("mode", ['setting', 'flag', 'flag-double', 'flag-shadow'])
+@pytest.mark.parametrize("setting_value", [True, False])
+@pytest.mark.parametrize("external_only", [True, False])
+@pytest.mark.parametrize("running_clients_num", [3])
+def test_focus_internal_external(hlwm, mode, setting_value, external_only, running_clients):
+    hlwm.call(f'set default_direction_external_only {hlwm.bool(setting_value)}')
+    layout = f"""
+    (split vertical:0.5:0
+        (clients vertical:0 {running_clients[0]} {running_clients[1]})
+        (clients vertical:0 {running_clients[2]}))
+    """.replace('\n', ' ').strip()
+    hlwm.call(['load', layout])
+    assert hlwm.get_attr('clients.focus.winid') == running_clients[0]
+
+    # we will now run 'focus' 'down' with -i or -e
+    cmd = ['focus']
+    if mode == 'setting':
+        hlwm.call(f'set default_direction_external_only {hlwm.bool(external_only)}')
+    else:
+        # mode is one of the flag*-modes
+        extonly2flag = {
+            True: '-e',
+            False: '-i',
+        }
+        if mode == 'flag-shadow':
+            # something like: focus -i -e down
+            # Here, an earlier and contradictory flag gets shadowed
+            cmd.append(extonly2flag[not external_only])
+        if mode == 'flag-double':
+            cmd.append(extonly2flag[external_only])
+        # the significant command line flag:
+        cmd.append(extonly2flag[external_only])
+    cmd += ['down']
+    hlwm.call(cmd)
+
+    if external_only:
+        expected_new_focus = running_clients[2]
+    else:
+        expected_new_focus = running_clients[1]
+    assert hlwm.get_attr('clients.focus.winid') == expected_new_focus
+
+
+def test_argparse_invalid_flag(hlwm):
+    # here, '-v' is not a valid flag, so it is assumed
+    # to be the first positional argument
+    hlwm.call_xfail('focus -v down') \
+        .expect_stderr('Cannot parse argument "-v"')
+    # in the following, the positional argument has been
+    # parsed already, so the error message is different:
+    hlwm.call_xfail('focus down -v') \
+        .expect_stderr('Unknown argument or flag "-v"')
+    hlwm.call_xfail('focus down -v -i') \
+        .expect_stderr('Unknown argument or flag "-v"')
+    hlwm.call_xfail('focus down -i -v') \
+        .expect_stderr('Unknown argument or flag "-v"')
+
+
 @pytest.mark.parametrize("path", '@ 0 1 00 11 . / /. ./'.split(' '))
 @pytest.mark.parametrize("running_clients_num", [3])
 @pytest.mark.parametrize("num_splits", [0, 1, 2])
@@ -230,14 +328,29 @@ def test_cycle(hlwm, running_clients, running_clients_num, num_splits, cycle_del
     new_windex = int(hlwm.get_attr('tags.0.curframe_windex'))
     expected_index = (windex + cycle_delta + wcount) % wcount if wcount > 0 else 0
     assert expected_index == new_windex
+    # check that the right winid is focused
+    layout = hlwm.call(['dump', '', '@']).stdout
+    winids = re.findall('0x[a-fA-f0-9]*', layout)
+    if len(winids) > 0:
+        assert winids[new_windex] == hlwm.get_attr('clients.focus.winid')
 
 
 @pytest.mark.parametrize("running_clients_num", [0, 1, 5])
 @pytest.mark.parametrize("index", [0, 1, 3, 5])
 def test_focus_nth(hlwm, running_clients, running_clients_num, index):
+    # bring the clients in the right order
+    layout = '(clients vertical:0 '
+    layout += ' '.join(running_clients)
+    layout += ')'
+    hlwm.call(['load', layout])
+
+    # focus the n_th
     hlwm.call('focus_nth {}'.format(index))
+
     windex = int(hlwm.get_attr('tags.0.curframe_windex'))
     assert windex == max(0, min(index, running_clients_num - 1))
+    if running_clients_num > 0:
+        assert hlwm.get_attr('clients.focus.winid') == running_clients[windex]
 
 
 @pytest.mark.parametrize("running_clients_num", [5])
@@ -330,6 +443,19 @@ def test_cycle_all_traverses_all(hlwm, running_clients, num_splits, delta):
     assert all_winids == visited_winids
 
 
+def test_cycle_all_errors(hlwm):
+    hlwm.call_xfail('cycle_all 1 2') \
+        .expect_stderr('Unknown argument or flag "2"')
+    hlwm.call_xfail('cycle_all -3') \
+        .expect_stderr('argument out of range')
+    hlwm.call_xfail('cycle_all 3') \
+        .expect_stderr('argument out of range')
+    hlwm.call_xfail('cycle_all -s 1') \
+        .expect_stderr('Cannot.*"-s"')
+    hlwm.call_xfail('cycle_all --skip-invisible -s 1') \
+        .expect_stderr('Cannot.*"-s"')
+
+
 @pytest.mark.parametrize("running_clients_num", [4])
 @pytest.mark.parametrize("delta", [1, -1])
 def test_cycle_all_skip_invisible(hlwm, running_clients, delta):
@@ -395,6 +521,53 @@ def test_cycle_all_with_floating_clients(hlwm, delta, floating_clients):
         hlwm.call(['cycle_all', delta])
 
 
+@pytest.mark.parametrize("delta", [1, -1])
+@pytest.mark.parametrize("clients_minimized", [
+    # every entry here is a list that determines the number
+    # of floating clients and which of those are minimized.
+    [],  # no floating clients at all
+    [True],  # only one, but it's minimized
+    [False],  # also covered by test_cycle_all_with_floating_clients
+    [False, True, False],  # one minimized in the middle
+    [True, True],
+    [False, True, True, False],
+    [True, True, True],
+    [True, False, True, False, True],
+    [False, False, True],
+    [True, False, False],
+    # multiple skips before/after wrapping between floating and tiling layer:
+    [True, True, False, False],
+    [False, False, True, True],
+])
+def test_cycle_all_skips_minimized(hlwm, delta, clients_minimized):
+    # create one tiled client and a list of floating clients,
+    # some of which are minimized.
+    tiled_client, _ = hlwm.create_client()
+    hlwm.call('rule floating=on')
+    non_minimized_clients = []
+    for minimized in clients_minimized:
+        winid, _ = hlwm.create_client()
+        hlwm.call(f'set_attr clients.{winid}.minimized {hlwm.bool(minimized)}')
+        if not minimized:
+            non_minimized_clients.append(winid)
+    if delta == -1:
+        non_minimized_clients = list(reversed(non_minimized_clients))
+    expected_winids = non_minimized_clients + [tiled_client]
+
+    assert hlwm.get_attr('clients.focus.winid') == tiled_client
+    print("expecting the clients in the order {}".format(' '.join(expected_winids)))
+    for expected in expected_winids:
+        # run 'cycle_all' as often as specified by 'expected_winids'.
+        hlwm.call(['cycle_all', delta])
+
+        # since there is a tiled client, clients.focus must always exist.
+        # we check that the focused client must never
+        # be minimized and must always be visible
+        assert hlwm.get_attr('clients.focus.minimized') == 'false'
+        assert hlwm.get_attr('clients.focus.visible') == 'true'
+        assert hlwm.get_attr('clients.focus.winid') == expected
+
+
 @pytest.mark.parametrize("running_clients_num", [2, 5])
 @pytest.mark.parametrize("num_splits", [0, 1, 2, 3])
 @pytest.mark.parametrize("delta", [1, -1])
@@ -792,26 +965,35 @@ def test_focus_other_monitor(hlwm, other_mon_exists, setting):
 
 def test_set_layout_invalid_layout_name(hlwm):
     hlwm.call_xfail('set_layout foobar') \
-        .expect_stderr('Invalid layout name: "foobar"')
+        .expect_stderr('Cannot.* "foobar":.*one of: vertical, horizontal')
 
 
-def test_focus_edge(hlwm):
+@pytest.mark.parametrize("command", ['focus_edge', 'shift_edge'])
+def test_focus_edge_shift_edge(hlwm, command):
     hlwm.call('set focus_crosses_monitor_boundaries on')
     hlwm.call('add otherTag')
     hlwm.call('add_monitor 800x600+800+0')
     hlwm.call('split right')
     hlwm.call('split right')
 
+    if command == 'shift_edge':
+        # the only difference between 'shift_edge' and 'focus_edge'
+        # is that we take the focused window with us
+        hlwm.create_client()
+
     # we're on the leftmost frame
     layout_before = hlwm.call('dump').stdout
-    hlwm.call('focus_edge left')
+    hlwm.call([command, 'left'])
+    assert hlwm.attr.tags.focus.tiling.focused_frame.index() == '00'
     assert layout_before == hlwm.call('dump').stdout
     assert hlwm.get_attr('monitors.focus.index') == '0'
 
-    # focus_edge goes to the rightmost frame
-    hlwm.call('focus_edge right')
+    # focus_edge/shift_edge goes to the rightmost frame
+    hlwm.call([command, 'right'])
     # we're still on the first monitor
     assert hlwm.get_attr('monitors.focus.index') == '0'
+    # but in the right-most frame
+    assert hlwm.attr.tags.focus.tiling.focused_frame.index() == '1'
     # but right-most frame means, if we go right once more, we're on
     # the other monitor:
     hlwm.call('focus right')
@@ -1013,3 +1195,167 @@ def test_frame_index_attribute(hlwm):
         # after each splitting operation, check that
         # the frame's index attribute is correct:
         verify_frame_tree('tags.focus.tiling.root', '')
+
+
+def test_focused_frame_child(hlwm):
+    # do it as one test case to see that the
+    # child correctly adjusts at run-time
+    test_data = [
+        ('', '(clients max:0)'),
+        ('0', '(split vertical:0.5:0 (clients max:0) (clients max:0))'),
+        ('1', '(split vertical:0.5:1 (clients max:0) (clients max:0))'),
+        ('00', '(split vertical:0.5:0 (split vertical:0.5:0) (clients max:0))'),
+        ('01', '(split vertical:0.5:0 (split vertical:0.5:1) (clients max:0))'),
+    ]
+    for focused_index, layout in test_data:
+        hlwm.call(['load', layout])
+        assert hlwm.get_attr('tags.0.tiling.focused_frame.index') == focused_index
+
+
+@pytest.mark.parametrize("old,new", [
+    ('(split vertical:0.4:0 {a} {b})', '(split vertical:0.6:1 {b} {a})'),
+    ('(split horizontal:0.3:0 {a} {b})', '(split horizontal:0.7:1 {b} {a})'),
+    ('(split vertical:0.5:1 {a} {b})', '(split vertical:0.5:0 {b} {a})'),
+    ('(split horizontal:0.2:1 {a} {b})', '(split horizontal:0.8:0 {b} {a})'),
+])
+@pytest.mark.parametrize("mode", ['horizontal', 'vertical'])
+def test_mirror_horizontal_or_vertical_one_split(hlwm, old, new, mode):
+    frame_a = '(clients horizontal:0)'
+    frame_b = '(clients vertical:0)'
+    hlwm.call(['load', old.format(a=frame_a, b=frame_b)])
+
+    hlwm.call(f'mirror {mode}')
+
+    if old.find(mode) < 0:
+        expected = old.format(a=frame_a, b=frame_b)  # nothing changes
+    else:
+        expected = new.format(a=frame_a, b=frame_b)
+    assert hlwm.call('dump').stdout == expected
+
+
+@pytest.mark.parametrize("num_splits", [1, 2, 3, 4])
+def test_mirror_vs_rotate(hlwm, num_splits):
+    for _ in range(0, num_splits):
+        hlwm.call('split explode 0.4')
+
+    layout = hlwm.call('dump').stdout
+
+    # compute the effect of rotating by 180 degrees
+    hlwm.call('chain , rotate , rotate')
+    layout_after_rotate = hlwm.call('dump').stdout
+    # restore original layout
+    hlwm.call(['load', layout])
+
+    # rotating by 180 degrees is the same as
+    # flipping both horizontally and vertically
+    hlwm.call('mirror both')
+    assert layout_after_rotate == hlwm.call('dump').stdout
+
+
+@pytest.mark.parametrize("direction, frameindex", [
+    ('left', '00'),
+    ('right', '1'),
+    ('up', '0100'),
+    ('down', '011'),
+])
+@pytest.mark.parametrize("clients_per_frame", [0, 1, 2])  # client count in other frame
+def test_shift_to_other_frame(hlwm, direction, frameindex, clients_per_frame):
+    """
+    in a frame grid with 3 columns, where the middle column has 3 rows, we put
+    the focused window in the middle, and then invoke 'shift' with the given
+    'direction'. Then, it is checked that the window stays focused but now
+    resides in the frame with the given 'frameindex'
+    """
+    winid, _ = hlwm.create_client()
+
+    def otherclients():
+        # put 'otherclients'-many clients in every other frame
+        winids = hlwm.create_clients(clients_per_frame)
+        return ' '.join(winids)
+
+    layout_131 = f"""
+    (split horizontal:0.66:0
+        (split horizontal:0.5:1
+            (clients vertical:0 {otherclients()})
+            (split vertical:0.66:0
+                (split vertical:0.5:1
+                    (clients vertical:0 {otherclients()})
+                    (clients vertical:0 {winid}))
+                (clients vertical:0 {otherclients()})))
+        (clients vertical:0 {otherclients()}))
+    """
+    hlwm.call(['load', layout_131])
+    assert hlwm.attr.clients.focus.winid() == winid
+    assert hlwm.attr.tags.focus.tiling.focused_frame.index() == '0101'
+
+    hlwm.call(['shift', direction])
+
+    # the window is still focused
+    assert hlwm.attr.clients.focus.winid() == winid
+    # but it's now in another frame
+    assert hlwm.attr.tags.focus.tiling.focused_frame.index() == frameindex
+
+
+@pytest.mark.parametrize("flag", [None, '-i', '-e'])
+@pytest.mark.parametrize("setting", ['on', 'off'])
+@pytest.mark.parametrize("running_clients_num", [1, 2, 4])
+def test_shift_internal_vs_external(hlwm, flag, setting, running_clients):
+    winids = running_clients  # define a shorter name for the slicing later
+
+    def layout(upper_winids, lower_winids, focus_idx):
+        # 'focus_idx' is the index of the focused window in
+        # the list 'upper_winids + lower_winids'
+        if focus_idx < len(upper_winids):
+            frame_selected = 0
+            upper_idx = focus_idx
+            lower_idx = 0
+        else:
+            frame_selected = 1
+            upper_idx = 0
+            lower_idx = focus_idx - len(upper_winids)
+        return normalize_layout_string(f"""
+        (split vertical:0.5:{frame_selected}
+            (clients vertical:{upper_idx} {' '.join(upper_winids)})
+            (clients vertical:{lower_idx} {' '.join(lower_winids)}))
+        """)
+    # put all clients in the upper frame
+    hlwm.call(['load', layout(winids, [], 0)])
+    assert hlwm.attr.clients.focus.winid() == winids[0]
+    hlwm.attr.settings.default_direction_external_only = setting
+
+    if flag == '-e' or (flag is None and setting == 'on') or len(winids) == 1:
+        # client is moved do another frame
+        expected_layout = layout(winids[1:], [winids[0]], len(winids[1:]))
+    else:
+        # client is moved within frame
+        expected_layout = layout(winids[1:2] + [winids[0]] + winids[2:], [], 1)
+
+    # shift the window
+    cmd = ['shift']
+    cmd += [flag] if flag is not None else []
+    cmd += ['down']
+    hlwm.call(cmd)
+
+    # the same client is still focused
+    assert hlwm.attr.clients.focus.winid() == winids[0]
+    assert hlwm.call(['dump']).stdout == expected_layout
+
+
+def test_shift_no_client_focused(hlwm):
+    hlwm.call('split vertical')
+
+    hlwm.call_xfail('shift down') \
+        .expect_stderr('No client focused')
+
+
+@pytest.mark.parametrize("split", [True, False])
+def test_shift_no_neighbour_frame(hlwm, split):
+    if split:
+        # splitting 'vertical' does not make a difference
+        # because it will focus the first frame
+        hlwm.call('split vertical')
+
+    hlwm.create_client()
+
+    hlwm.call_xfail('shift up') \
+        .expect_stderr('No neighbour found')
